/*****************************************************************************
 * Create the SAS provided environment containing common models, 
 * functions and environment configuration.
 *
 * A 'base' environment will be created from this one that the user can 
 * customize. 
 ****************************************************************************/
%let env_name = mimp_sas.sas_risk_env;

proc risk; 
  env new=&env_name 
      label="SAS provided environment containing common models, functions and environment configuration";
  env save;
run;
quit;

proc compile env= &env_name
             outlib= &env_name
             package = funcs;

  /*- create vector version of rantbl -*/
  function rantbl_array( vec[*]);  
     sum = 0;
     draw = rand('uniform');
     to_state=.;
     do index = 1 to dim(vec);
          if vec[index]=. then return (.);
          sum = sum + vec[index];
          if (sum>draw and missing(to_state)) then to_state=index;                       
     end;
     if (abs(sum-1.0) > 0.000000001) then return (.);
     else return(to_state);
  endsub;


  /*- create model result structure instance -*/
  method base_project kind= project;
     beginblock thread_init;
        
     endblock;
  endmethod;

run;
quit;



proc compile env=&env_name outlib=&env_name package=funcs;



  /*---------------------------------------------------------*/ 
  /* Name:      mip_pwl1d                                    */ 
  /*                                                         */ 
  /* Purpose:   Piece-wise Linear 1D interpolation           */ 
  /*                                                         */ 
  /* Inputs:                                                 */ 
  /*   knots - an array of knot values                       */ 
  /*   betas - an array of beta values                       */ 
  /*   value - a value relative to knots that is used to     */ 
  /*           calculate the returned weighted beta value.   */ 
  /*                                                         */ 
  /* Returns:  a weighted beta value                         */ 
  /*---------------------------------------------------------*/ 
  function mip_pwl1d(knots[*], betas[*], value); 
      startKnot=2; 
 
      if (value=.) then do; 
        if (missing(knots[1])) then return (betas[1]); 
          else return (.);  /* We need to error here */ 
      end; 
      else do; 
        if (missing(knots[1])) then startKnot+1; 
        if (value <= knots[startKnot-1]) then 
          return (betas[startKnot-1]);       /* when prior to range of knots */ 
 
      n = dim(knots); 
      do i=startKnot to n; 
        if (value <= knots[i]) then 
        do; 
          weight=(knots[i]-value) / (knots[i]-knots[i-1]); 
          return ( weight*betas[i-1] + (1.0-weight)*betas[i] ); 
        end; 
      end; 
    end; 
 
    return (betas[n]);     /* when beyond range of knots */ 
 
  endsub; 

  /*---------------------------------------------------------*/ 
  /* Name:      mip_pwl1d_util                              */ 
  /*                                                         */ 
  /* Purpose:   Piece-wise Linear 1D interpolation utility   */ 
  /*            used for calculating the weights and indices */ 
  /*            for PWL2D interpolation.                     */ 
  /*                                                         */ 
  /* Inputs:                                                 */ 
  /*   knots  - an array of knot values                      */ 
  /*   value  - a value relative to knots that is used to    */ 
  /*            calculate the weight and index               */ 
  /*                                                         */ 
  /* Returns (via output args):  a weight and (upper) index  */ 
  /*---------------------------------------------------------*/ 
  subroutine mip_pwl1d_util(knots[*], value, out_weight, out_index); 
    outargs out_weight, out_index; /* returned arguments */
    startKnot=2; 
 
    if (value=.) then do; 
      if (missing(knots[1])) then 
      do; 
        out_weight=1.0; 
        out_index=2; 
        return; 
      end; 
      else do; 
        out_weight=.; 
        out_index=2; 
        return;  /* We need to error here */ 
      end; 
    end; 
    else do; 
      if (missing(knots[1])) then startKnot+1; 
        if (value <= knots[startKnot-1]) then 
        do; 
          out_weight=1.0; 
          out_index=startKnot; 
          return; 
        end; 
 
        n = dim(knots); 
        do i=startKnot to n; 
          if (value <= knots[i]) then 
          do; 
            out_weight=(knots[i]-value) / (knots[i]-knots[i-1]); 
            out_index=i; 
            return; 
          end; 
      end; 
    end; 
 
    out_weight=0; 
    out_index=n; 
    return; 
  endsub; 


  /*---------------------------------------------------------*/ 
  /* Name:      mip_pwl2d                                    */ 
  /*                                                         */ 
  /* Purpose:   Piece-wise Linear 2D interpolation           */ 
  /*            using a PWL 1D utility function for          */ 
  /*            calculating weights and indices.             */ 
  /*                                                         */ 
  /* Inputs:                                                 */ 
  /*   knots1 - an array of knot values for 1st dimension    */ 
  /*   knots2 - an array of knot values for 2nd dimension    */ 
  /*   betas  - a 2D array of beta values                    */ 
  /*   value1 - a value relative to knots1 that is used to   */ 
  /*           calculate the weight for 1st dimension        */ 
  /*   value2 - a value relative to knots2 that is used to   */ 
  /*           calculate the weight for 2nd dimension        */ 
  /*                                                         */ 
  /* Returns:  a weighted beta value                         */ 
  /*---------------------------------------------------------*/ 
  function mip_pwl2d(knots1[*], knots2[*], betas[*,*], value1, value2);   
    wt1=0; 
    wt2=0; 
    i=0; 
    j=0; 
    call mip_pwl1d_util(knots1, value1, wt1, i); 
    call mip_pwl1d_util(knots2, value2, wt2, j); 
    return ( betas[i-1, j-1] *  wt1    * wt2 + 
             betas[i-1, j  ] *  wt1    * (1-wt2) + 
             betas[i  , j-1] * (1-wt1) * wt2 + 
             betas[i  , j  ] * (1-wt1) * (1-wt2) ); 
  endsub; 


run;
quit;



