/***********************************************************************
 * Copyright 2015, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:          rskmimpl_rest_get_mdlgrp_code
 *
 * PURPOSE:       Get model groups code from RISK Model Implementation Platform REST API. 
 *
 * PARAMETERS: 
 *              MIM_HOST 
 *                  <required> - The server where MIMP is running.
 *              MIM_PORT 
 *                  <required> - The REST API PORT 
 *              MIM_USERNAME 
 *                  <required> - The user name to access the REST API.
 *              MIM_PASSWORD 
 *                  <required> - The password for accessing the sever REST API.
 *              MIM_RISK_ENV 
 *                  <required> - Specifies the Risk environment where generated model group code will be compiled. 
 *              MIM_MODEL_GROUPS_DS
 *                  <required> - Specifies a SAS data set that contains model groups. It has to have model_group column.
 *              MIM_RISK_FACTOR_DS
 *                  <optional> - Specifies a date set that declares risk factor variables in Risk environment. It has to have 
 *                               columns NAME and KIND.
 *                  if not specified, defaults to blank. 
 *              MIM_OUTLIB 
 *                  <optional> - Specifies the library for output data sets 
 *                  if not specified, defaults to WORK.
 *              MIM_OUTVAR_DS 
 *                  <optional> - Specifies the output data set to declare output variables in Risk environment. 
 *                  if not specified, defaults to mim_outvar_ds.
 *              MIM_MODEL_GROUP_MODEL_MAPPING_DS
 *                  <optional> - Specifies the output data set that contains model names and methods for model groups.
 *                  if not specified, defaults to mim_model_group_model_mapping_ds.  
 *              MIM_CODE_FILE
 *                  <optional> - Specifies the complete path and filename or a fileref for the the generated SAS code file.
 *                               If you specify a fileref, you should omit the quotation marks.
 *                  if not specified, the generated code will be stored in rskmimpl_model_groups_model_code.sas in work library. 
 *              MIM_REQUIRE_TICKET 
 *                  <optional> - Specifies if a ticket is needed for accessing the server REST API. 
 *                  if not specified, defaults to Y.
 *              MIM_SUBMIT_CODE 
 *                  <optional> - Specifies if the generated code is submitted.
 *                  if not specified, defaults to Y.
 *
 * EXAMPLE: %rskmimpl_rest_get_mdlgrp_code(MIM_HOST=localhost, 
 *                                         MIM_PORT=8080, 
 *                                         MIM_USERNAME=sas, 
 *                                         MIM_PASSWORD=sas, 
 *                                         MIM_RISK_ENV=test,
 *                                         MIM_MODEL_GROUPS_DS=model_group_mapping_ds,
 *                                         MIM_RISK_FACTOR_DS=riskvar_ds,
 *                                         MIM_OUTLIB=WORK,
 *                                         MIM_OUTVAR_DS=MIM_OUTVAR_DS,
 *                                         MIM_MODEL_GROUP_MODEL_MAPPING_DS=MIM_MODEL_GROUP_MODEL_MAPPING_DS,
 *                                         MIM_CODE_FILE="C:\Work\Test\mim_test_generated_code.sas",
 *                                         MIM_REQUIRE_TICKET=Y,
 *                                         MIM_SUBMIT_CODE=Y
 *                                         );
 ***********************************************************************/

%macro  rskmimpl_rest_get_mdlgrp_code(MIM_HOST=, 
                                      MIM_PORT=, 
                                      MIM_USERNAME=, 
                                      MIM_PASSWORD=, 
                                      MIM_RISK_ENV=,
                                      MIM_MODEL_GROUPS_DS=,
                                      MIM_RISK_FACTOR_DS=,
                                      MIM_OUTLIB=WORK,
                                      MIM_OUTVAR_DS=MIM_OUTVAR_DS,
                                      MIM_MODEL_GROUP_MODEL_MAPPING_DS=MIM_MODEL_GROUP_MODEL_MAPPING_DS,
                                      MIM_CODE_FILE= "",
                                      MIM_REQUIRE_TICKET=Y,
                                      MIM_SUBMIT_CODE=Y
                                      );

    %local i service_ticket path_dlm user_tmp_folder 
           model_group_exist max_model_group_length number_model_groups 
           http_status_created;

    %let url=%str(http://&mim_host:&mim_port/SASModelImplementationPlatform/rest/modelGroups/batchCode);

    %if %index(WIN,&sysscp) or %index(DNTHOST,&sysscp) %then %let path_dlm=\;                                                                                                     
    %else %let path_dlm=/;    

    %if %length(&MIM_HOST) eq 0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The parameter MIM_HOST cannot be blank.;
        %return;
    %end;

    %if %length(&MIM_PORT) eq 0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The parameter MIM_PORT cannot be blank.;
        %return;
    %end;

    %if %length(&MIM_RISK_ENV) eq 0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The parameter MIM_RISK_ENV cannot be blank.;
        %return;
    %end;

    %if %length(&MIM_MODEL_GROUPS_DS) eq 0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The parameter MIM_MODEL_GROUPS_DS cannot be blank.;
        %return;
    %end;
    %if not %index(&MIM_MODEL_GROUPS_DS, .) %then %let MIM_MODEL_GROUPS_DS=work.&MIM_MODEL_GROUPS_DS;

    %if %lowcase(&MIM_REQUIRE_TICKET) eq y %then %do;
        %rskmimpl_rest_service_ticket(mim_url=&url, 
                                      mim_username=&MIM_USERNAME, 
                                      mim_password=&MIM_PASSWORD, 
                                      mim_service=SASModelImplementationPlatform,
                                      mim_service_ticket_macro_var=service_ticket
                                      );

        %if %index(%str(&service_ticket), %str(ST-)) NE 1 %then %do;
            %put ERROR:(rskmimpl_rest_get_mdlgrp_code) Can not retrieve model groups code because it failed to get the service ticket for REST API.;
            %return;
        %end;
    %end;

    %let max_model_group_length=0;
    %let number_model_groups=0;
    proc contents noprint data=&MIM_MODEL_GROUPS_DS out=_rrgmc_model_groups_contents;
    run;

    proc sql noprint;
        select count(*), LENGTH, NOBS into :model_group_exist, :max_model_group_length, :number_model_groups
        from _rrgmc_model_groups_contents
        where lowcase(NAME)='model_group';
    quit;

    %if &model_group_exist=0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) MODEL_GROUP does not exist in %upcase(&MIM_MODEL_GROUPS_DS).;
        %return;
    %end;

    %if &number_model_groups=0 %then %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The data set %upcase(&MIM_MODEL_GROUPS_DS) is empty.;
        %return;
    %end;

    %let max_model_group_length=%sysfunc(max(256,&max_model_group_length+20));
    %let number_model_groups=&number_model_groups;

    %let user_tmp_folder = %sysfunc(pathname(work));

    %if %length(&MIM_CODE_FILE) eq 0 %then %do;
        filename _outfile "&user_tmp_folder&path_dlm.rskmimpl_model_groups_model_code.sas";
    %end;
    %else %if %length(%sysfunc(translate(&MIM_CODE_FILE,%str(  ),%str(%'%")))) eq 0 %then %do;
        filename _outfile "&user_tmp_folder&path_dlm.rskmimpl_model_groups_model_code.sas";
    %end;
    %else %if %sysfunc(prxmatch([^%str(%').+%str(%')$|^%str(%").+%str(%")$], &MIM_CODE_FILE)) gt 0 %then %do;
        filename _outfile &MIM_CODE_FILE;
    %end;
    %else %if %length(&MIM_CODE_FILE) le 8 %then %do;
        filename _outfile "%sysfunc(pathname(&MIM_CODE_FILE))";
    %end;
    %else %do;
        %put ERROR:(rskmimpl_rest_get_mdlgrp_code) The fileref &MIM_CODE_FILE exceeds 8 characters.;
        %return;
    %end;

    filename _hin "&user_tmp_folder.&path_dlm._hin.txt";
    filename _hout "&user_tmp_folder.&path_dlm._hout.txt";
    filename _in "&user_tmp_folder.&path_dlm._in.txt";
    filename _out "&user_tmp_folder.&path_dlm._out.txt";

    /*Create HTTP request header file*/
    data _null_;
        file _hin;
        put 'accept: application/json';
    run;

    proc sort data=&MIM_MODEL_GROUPS_DS(keep=model_group) out=_rrgmc_model_groups nodupkey;
        by model_group;
    run;

    data _null_;
        call symput('number_model_groups', nobs);
        stop;
        set _rrgmc_model_groups nobs=nobs;
    run;
    %let number_model_groups=&number_model_groups;
    
    data _null_;
        file _in lrecl=&max_model_group_length;
        set _rrgmc_model_groups(in=_in_mdlgrp keep=model_group) 
            %if %length(&mim_risk_factor_ds) > 0 %then %do;
                &MIM_RISK_FACTOR_DS(in=_in_rf keep=name kind where=(upcase(kind)="FACTOR")) 
            %end;
            end=eof;
        
        if _n_=1 then do;
            put '{';
            put ' "riskEnviroment":"' "&MIM_RISK_ENV" '",';
            put ' "outLib":"' "&MIM_OUTLIB" '",';
            %if %length(&MIM_OUTVAR_DS) gt 0 %then put ' "outVarDSName":"' "&MIM_OUTVAR_DS" '",';
            %else put ' "outVarDSName":"",';;
            %if %length(&MIM_MODEL_GROUP_MODEL_MAPPING_DS) gt 0 %then put ' "modelDSName":"' "&MIM_MODEL_GROUP_MODEL_MAPPING_DS" '",';
            %else put ' "modelDSName":"",';;
            put ' "modelGroups":[';
        end;

        if _in_mdlgrp then do;
            put_str=cats('"',model_group,'"');
            if _n_ < &number_model_groups then put_str=cats(put_str,',');
            put @17 put_str;
            if _n_ = &number_model_groups then put @16 '],';
        end;

        %if %length(&mim_risk_factor_ds) > 0 %then %do;
            if _in_rf then do;
                if _n_ = &prdt_mdl_grp_mapping_number+1 then put ' "riskVariables":[';
                put @19 '{';
                put_str=cats('"variableName":"',name,'",');
                put @20 put_str;
                put @20 '"riskVarType":"riskFactor",';
                put @20 '"riskVarTypeName":""';
                if not eof then put @19 '},';
                else do;
                    put @19 '}';
                    put @18 '],';
                end;
            end;
            else if eof then put ' "riskVariables": null,';
        %end;
        %else %do;
            if eof then put ' "riskVariables": null,';
        %end;

        if eof then do;
            put ' "code":null';
            put '}';
        end;
    run;

    proc http url="&url?ticket=&service_ticket" 
              headerin=_hin 
              headerout=_hout
              in=_in
              out=_out
              method='post' 
              CT="Application/json"
              ;
    run;
    
    /*Check HTTP status code*/
    %let http_status_created=Y;
    data _null_;
        infile _hout;
        input;
        retain output_msg 0;

        if _n_=1 and strip(_infile_) ne "HTTP/1.1 201 Created" then do;
            call symput('http_status_created', 'N');
            put "ERROR:(rskmimpl_rest_get_mdlgrp_code) Error report:";
            output_msg=1;
        end;

        if output_msg then put _infile_;
    run;

    %if &http_status_created eq Y %then %do;
        /*Parsing the return JSON object to get generated SAS code*/
        data _null_;
            file "&user_tmp_folder.&path_dlm.parse_sas_code.lua";

            put "local infile=sas.pathname('_out')";
            put "local outfile=sas.pathname('_outfile')";

            put "local inf=assert(io.open(infile, 'r'))";
            put "local t=inf:read('*all')";
                 
            put "local json_item_names={'""jobName"":','""riskEnviroment"":','""outLib"":','""outVarDSName"":','""modelDSName"":','""modelGroupMapping"":','""riskVariables"":','""code"":'}";

            put "local indx_code_start=string.find(t,json_item_names[8]..'%nrstr(%s)*""')";
            put "if indx_code_start == Nil then";
            put "    error('No SAS code is generated in the return JSON file from the REST API.')";
            put "end";

            put "t=string.sub(t,indx_code_start+json_item_names[8]:len())";
            put "local indx_code_end=string.find(t,'%nrstr(%s)*}%nrstr(%s)*$')";
            put "for i=1, 7 do";
            put "   local indx=string.find(t,"",%nrstr(%s)*""..json_item_names[i])";
            put "   if indx ~= Nil and indx < indx_code_end then";
            put "       indx_code_end=indx";
            put "   end";
            put "end";

            put "t=string.sub(t,1,indx_code_end-1)";
            
            put "local dlm=string.char(13)..string.char(10)";
            put "local tab=string.char(9)";

            put "t=t:gsub('^%nrstr(%s)*""', ''):gsub('%nrstr(%s)*""%nrstr(%s)*$',''):gsub('\\""','""'):gsub('\\r\\n',dlm):gsub('\\n',dlm):gsub('\\t',tab)";
            put "local outf=assert(io.open(outfile, 'w'))";
            put "outf:write(t)";
        run;

        filename LuaPath "&user_tmp_folder";
        proc lua restart infile='parse_sas_code';
        run;

        %if &SYSERR eq 0 and %upcase(&MIM_SUBMIT_CODE) eq Y %then %do;
            %inc _outfile;
        %end;
    %end;

    filename _hin clear;
    filename _hout clear;
    filename _in clear;
    filename _out clear;
    filename _outfile clear;
%mend rskmimpl_rest_get_mdlgrp_code;
