/***********************************************************************
 * Copyright 2015, SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 * NAME:          rskmimpl_runmodels
 *
 * PURPOSE:       Run models on a pre-existing risk cube.
 *                Produces summary and scendata datasets for top-level
 *                and InstType queries.
 *
 * PARAMETERS: 
 *              MIM_GET_SWC_PROPERTIES 
 *                  <optional> - defaults to YES: get HPRISK SWC properties from metadata.
 *              MIM_PRIOR_CUBE_DESC_QUOTED 
 *                  <required> - The quoted full path name for the cube descriptor to create,
 *                  must be writable by the proc hprisk machine.
 *              MIM_CUBE_DESC_QUOTED 
 *                  <required> - The quoted full path name for the cube descriptor to create,
 *                  must be writable by the proc hprisk machine.
 *              MIM_GRID_PATH_QUOTED 
 *                  <required> - The quoted path to the gridfile folder for the cube to use.
 *                  must be a writable area in the grid.
 *              MIM_LOCAL_PATH_QUOTED 
 *                  <required> - The quoted path to the local folder for the cube to use.
 *                  must be a writable area on the sas server.
 *              MIM_PREFIX_NAME_QUOTED
 *                  <required> - The quoted name for cube files to use as a prefix.
 *                  It will be appended to either MIM_GRID_PATH_QUOTED or MIM_LOCAL_PATH_QUOTED.
 *              MIM_ENV 
 *                  <required> - The Risk Dimensions environment to export.
 *              MIM_PROJECT 
 *                  <optional> - The name of a Risk Dimensions project to export.
 *                  if not specified, defaults to all
 *              MIM_ENV_OVERRIDE_LIB 
 *                  <optional> - Specifies the location of the export libref
 *                               if we need to re-export. If not provided we will use
 *                               the one in the prior cube.
 *              MIM_CURDATA 
 *                  <optional> - libname.dataset for current economic data 
 *              MIM_HISTDATA 
 *                  <optional> - libname.dataset for historical economic data 
 *              MIM_SCENARIOS 
 *                  <required> - name of scenarios dataset. 
 *                               Will be run and query results produced.
 *              MIM_INSTTYPE_COL 
 *                  <optional> - name of insttype column for queries, defaults to InstType
 *              MIM_NUM_SIMULATIONS=
 *                  <optional> - number of monte-carlo simulations
 *              MIM_DEBUG=
 *                  <optional> - YES|NO, defaults to NO
 *              MIM_STATUSDS=
 *                  <optional> - status dataset
 *              MIM_HPUICFGDS=
 *                  <optional> - dataset containing run settings to allow
 *                               hprisk ui to display the cube later
 *              MIM_CLEAN_CUBE=
 *                  <optional> - YES|NO, defaults to NO , clean cube
 *                               upon completion of run
 *              MIM_OUTVARS=
 *                  <optional> - Specifies output variables
 *
 *  EXAMPLE: %rskmimpl_runmodels(
 *              MIM_GET_SWC_PROPERTIES=YES
 *              MIM_PRIOR_CUBE_DESC_QUOTED="/rqs/RskMim/cubes/MyCube1",
 *              MIM_CUBE_DESC_QUOTED="/rqs/RskMim/cubes/MyCube2",
 *              MIM_GRID_PATH_QUOTED="/grid/RskMim/gridfiles",
 *              MIM_LOCAL_PATH_QUOTED="/local/RskMim/gridfiles",
 *              MIM_PREFIX_NAME_QUOTED="MyCube",
 *              MIM_ENV=mimBase,
 *              MIM_PROJECT=MIPProject, 
 *              MIM_CURDATA=myresults.curdata,
 *              MIM_HISTDATA=myresults.histdata,
 *              MIM_SCENARIOS=libname.scenarios
 *              MIM_INSTTYPE_COL=InstType
 *              MIM_NUM_SIMULATIONS=100
 *              );
 ***********************************************************************/
%macro rskmimpl_runmodels( 
                               MIM_GET_SWC_PROPERTIES=      YES,
                               MIM_CUBE_DESC_QUOTED=        "",
                               MIM_GRID_PATH_QUOTED=        "",
                               MIM_LOCAL_PATH_QUOTED=       "",
                               MIM_PREFIX_NAME_QUOTED=      "",
                               MIM_PRIOR_CUBE_DESC_QUOTED=  "",
                               MIM_ENV_OVERRIDE_LIB=          ,
                               MIM_CURDATA=                   ,
                               MIM_HISTDATA=                  ,
                               MIM_SCENARIOS=                 ,
                               MIM_PORTDATA=                  ,
                               MIM_INSTTYPE_COL=            InstType,
                               MIM_ENV=                       ,
                               MIM_PROJECT=                   ,
                               MIM_NUM_SIMULATIONS=           ,
                               MIM_RESULTS_LIB=             work,
                               MIM_SCEN_RESNAME=            result,
                               MIM_OUTVARS=                   ,
                               MIM_DEBUG=                   NO,
                               MIM_STATUSDS=                miplogs.codeGenStatus,
                               MIM_HPUICFGDS=               mipjob._hprui_cfg,
                               MIM_CLEAN_CUBE=              NO 
                               );

    /*------------------------------------------------------*/
    /*--  basic validation                               ---*/
    /*------------------------------------------------------*/
    %local _unquoted_cubedesc;
    %let   _unquoted_cubedesc = %trim(%left(%sysfunc(dequote(&MIM_CUBE_DESC_QUOTED))));

    %if %sysfunc(fileexist(&_unquoted_cubedesc..rskcdesc)) %then %do;                
        %put ERROR:(rskmimpl_runmodels) The prior cube descriptor does not exists. Please specify a valid portfolio cube. ;
        %hpr_prog_ds_append(HPR_DS=&MIM_STATUSDS,  HPR_TASK_QUOTED="RUNMODELS",  HPR_STEP_DTTM=0, 
                            HPR_ACTION_QUOTED="RUNMODELS_ERR", 
                            HPR_DESC_QUOTED="The prior cube descriptor does not exist",  
                            HPR_STEP_CNT=1,  HPR_TOTAL_CNT=1,  HPR_FAIL=1);
        %return;
    %end;

    %if %sysfunc(fileexist(&_unquoted_cubedesc..rskcdesc)) %then %do;         
        %put ERROR:(rskmimpl_runmodels) The cube descriptor already exists. Remove the cube and try again. ;
        %hpr_prog_ds_append(HPR_DS=&MIM_STATUSDS,  HPR_TASK_QUOTED="RUNMODELS",  HPR_STEP_DTTM=0, 
                            HPR_ACTION_QUOTED="RUNMODELS_ERR", 
                            HPR_DESC_QUOTED="The cube descriptor already exists",  
                            HPR_STEP_CNT=1,  HPR_TOTAL_CNT=1,  HPR_FAIL=1);
        %return;
    %end;

    %if %sysfunc(exist(&MIM_SCENARIOS)) ne 1  %then %do; 
        %put ERROR:(rskmimpl_runmodels) No scenarios have been provided or the dataset does not exist. code= %sysfunc(exist(&MIM_SCENARIOS)) ;
        %hpr_prog_ds_append(HPR_DS=&MIM_STATUSDS,  HPR_TASK_QUOTED="RUNMODELS",  HPR_STEP_DTTM=0,
                            HPR_ACTION_QUOTED="RUNMODELS_ERR",   
                            HPR_DESC_QUOTED="No scenarios have been provided or the dataset does not exist",  
                            HPR_STEP_CNT=1,  HPR_TOTAL_CNT=1,  HPR_FAIL=1);
        %return;
    %end;   

    /*------------------------------------------------------*/
    /*--- STAGE 1: re-export project                     ---*/
    /*------------------------------------------------------*/
    %if &MIM_ENV_OVERRIDE_LIB ne %str() %then %do;
       proc hpexport
              env = &MIM_ENV
              project = &MIM_PROJECT
              expprojlib = &MIM_ENV_OVERRIDE_LIB;
       run; quit;

       %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;  

          %hpr_prog_ds_append(HPR_DS=&MIM_STATUSDS,  HPR_TASK_QUOTED="RUNMODELS",  HPR_STEP_DTTM=0,  
                            HPR_ACTION_QUOTED="RUNMODELS_ERR",   
                            HPR_DESC_QUOTED="The environment export failed.",
                            HPR_STEP_CNT=1,  HPR_TOTAL_CNT=1,  HPR_FAIL=&SYSERR);
     
          %return;
       %end;
    %end;

    %if &MIM_CLEAN_CUBE ne YES %then %do; 

      /*------------------------------------------------------*/ 
      /*---  write configuration to hp viewer dataset      ---*/ 
      /*------------------------------------------------------*/
      %local MIM_PATHSEP ;                                    
      %if %index(WIN,&sysscp) or %index(DNTHOST,&sysscp) %then %do;
        %let MIM_PATHSEP=\;
      %end;
      %else %do;  
        %let MIM_PATHSEP=/; 
      %end;

      %local MIM_SIMPLE_CUBE_NM ;                                    
      %let MIM_SIMPLE_CUBE_NM = %scan( &_unquoted_cubedesc,  -1, &MIM_PATHSEP, mo);
   
      data &MIM_HPUICFGDS;
         length name $32 value $1024; 
         name = "TYPE";  value="STRESS"; output;
         name = "PATH";  value=&MIM_CUBE_DESC_QUOTED; output;
         name = "NAME";  value="&MIM_SIMPLE_CUBE_NM"; output;
      run;
    

       /*------------------------------------------------------*/
       /*--- handle grid stuff/cube filenames               ---*/
       /*------------------------------------------------------*/
       %local IS_DISTRIBUTED;
       %hpr_get_swc_properties(HPR_GET_SWC_PROPERTIES=&MIM_GET_SWC_PROPERTIES,
                               HPR_DISTRIB_FLAG=IS_DISTRIBUTED);

       %local _unquoted_grid_path;
       %let _unquoted_grid_path = %trim(%left(%sysfunc(dequote(&MIM_GRID_PATH_QUOTED))));
       %local _unquoted_loc_path;
       %let _unquoted_loc_path = %trim(%left(%sysfunc(dequote(&MIM_LOCAL_PATH_QUOTED))));
       %local _unquoted_prefix_name;
       %let _unquoted_prefix_name = %trim(%left(%sysfunc(dequote(&MIM_PREFIX_NAME_QUOTED))));

       %local _result_cubedesc;
       %let _result_cubedesc = "&_unquoted_cubedesc.";

       %local _result_fileprefix;
       %if "&IS_DISTRIBUTED" eq "DISTRIBUTED" %then %do;
            %let _result_fileprefix = "&_unquoted_grid_path./&_unquoted_prefix_name";
       %end;
       %else %do;
            %let _result_fileprefix = "&_unquoted_loc_path./&_unquoted_prefix_name";
       %end;

    %end;

    /*------------------------------------------------------*/
    /*--- STAGE 2:re-run models (store results in cube)  ---*/
    /*------------------------------------------------------*/
    proc hprisk
        task           = stress        
        cube           = &MIM_PRIOR_CUBE_DESC_QUOTED

     %if &MIM_DEBUG eq YES %then %do;
        tracelog       = YES
     %end;

     %if &MIM_CURDATA ne %str() %then %do;
        currentdata    = &MIM_CURDATA
     %end;

     %if &MIM_HISTDATA ne %str() %then %do;
        hist_econ_data = &MIM_HISTDATA
     %end;

     %if &MIM_NUM_SIMULATIONS ne %str() %then %do;
        mcreps         = &MIM_NUM_SIMULATIONS         
     %end;

     %if &MIM_ENV_OVERRIDE_LIB ne %str() %then %do;
        expprojlib = &MIM_ENV_OVERRIDE_LIB
     %end;

     %if &MIM_STATUSDS ne %str() %then %do;
        progress = &MIM_STATUSDS
     %end;

        ;
        
     %if &MIM_CLEAN_CUBE ne YES %then %do;
        cube descriptor  = &_result_cubedesc  
             filesprefix = &_result_fileprefix
             resultset   = &MIM_SCEN_RESNAME;
     %end;

        scenarioanalysisdata  data= &MIM_SCENARIOS;

        output  scen    = &MIM_RESULTS_LIB..scenresults
                summary = &MIM_RESULTS_LIB..summary
                transmatbalancedata = &MIM_RESULTS_LIB..transbalance;

     %if %length(&MIM_OUTVARS) gt 0 %then %do;
        outvars add=( &MIM_OUTVARS );
     %end; 

        query;
        query &MIM_INSTTYPE_COL;

    run; quit;           
  
%mend rskmimpl_runmodels;
